function [alpha,beta,gamma,quadwts,quadvals1, quadvals2,converge]=em_npml_fridel(alphainit,betainit,gammainit,mquad)
% fits mixture of two exponential family GLMs each with random intercept by NPML


% glmlab globals;
global YVAR LINKFUNCTION XVARS ERRDISFUNCTION SCALEPAR INC_CONST;
global OFFSET WEIGHTS DETAILSFILE NAMEXV NAMELIST NAMEYV identity;

global Y ID ID2 N N2 XMAT ZMAT WMAT BINN1 BINN2 ERRDIST1 ERRDIST2 MU1LINK 
global MU2LINK PLINK OFFSET1 OFFSET2;


clear paramtrs;
[toler,maxits,illctol]=myparamtrs %load defaults
converge=0;

quadwtsinit =[   0.07718399665045   0.136462301   0.13888795910759   0.17504762830876   0.20555368975748   0.06733699007994   0.19952743383709]';
quadvals1init =[  -3.53232834327429  -4.36151188124223  -5.43777170255962  -6.27835886715718  -3.79456097829275  -2.26994797826750  -4.05724872099781]';
quadvals2init =[   3.27828414975511   3.35678302771616   1.73019687339642   3.17321509199899   2.91127651348920   3.34647540589581   1.68645207260083]';

identity=eye(mquad);


alpha=alphainit
beta=betainit
gamma=gammainit
quadwts=quadwtsinit
quadvals1=quadvals1init
quadvals2=quadvals2init


dimbeta=size(beta,1);
dimgamma=size(gamma,1);
dimalpha=size(alpha,1);
dimvals1=size(quadvals1,1);
dimvals2=size(quadvals2,1);
dimwts=size(quadwts,1)-1;
dimparm2=dimwts;
dimparm1=dimalpha+dimbeta+dimgamma+dimvals1+dimvals2;
dimparm=dimparm1+dimparm2;
its=0;
fail=0;

loglik= getloglik(alpha,beta,gamma,mquad,quadwts,quadvals1,quadvals2)

while (its<maxits)

  emitcount=0;
  while (emitcount<5000) % do EM algorithm 5 times
    emitcount=emitcount+1
    its=its+1
 
    alphaold=alpha; 
    betaold=beta;
    gammaold=gamma;
    quadwtsold=quadwts;
    quadvals1old=quadvals1;
    quadvals2old=quadvals2;


    alp=OFFSET1+XMAT*alpha;
    blp=OFFSET2+ZMAT*beta;
    glp=WMAT*gamma;
    p=getmixp(glp);
    
    %M step for quadwts (pi)

    fofyijcond=zeros(N,mquad);
    for l=1:mquad
        mu1cond=getmu1( alp + identity(l,:)*quadvals1,BINN1);
        mu2cond=getmu2( blp + identity(l,:)*quadvals2,BINN2);
        fofyijcond(:,l)= p.*getf1cond(Y,mu1cond,BINN1)+(1-p).*getf2cond(Y,mu2cond,BINN2);
    end;
    
    w=zeros(N2,mquad);
    for i=1:N2;
        ti=size(ID(ID==i),1);
        denom = prod(fofyijcond(ID==i,:))*quadwts;
        if (denom==0)
           w(i,:)=(zeros(mquad,1))';
        else
           w(i,:) =ti*(prod(fofyijcond(ID==i,:)).*(quadwts'))/denom;
        end;
    end;
    quadwtsnew = sum(w)/N;
    quadwtsnew=quadwtsnew



    %M step for gamma
    warning off
    newmodel; 
    clear global YVAR XVARS;
    global YVAR LINKFUNCTION XVARS ERRDISFUNCTION SCALEPAR INC_CONST;
    global OFFSET WEIGHTS DETAILSFILE NAMEXV NAMELIST NAMEYV;
    warning on
    NAMEXV='[';
    for i=1:size(WMAT,2)
      NAMEXV=strcat(NAMEXV,'G'); 
      NAMEXV=strcat(NAMEXV,int2str(i));
      if (i~=size(WMAT,2))
        NAMEXV=strcat(NAMEXV,',');
      end;
    end;
    NAMEXV=strcat(NAMEXV,']');
    NAMEYV='[y]';
    NAMELIST=['Constant'];
    for i=1:size(WMAT,2)-1
      NAMELIST=str2mat(NAMELIST,['Gvar ',num2str(i)]);
    end;

    INC_CONST=0;
  
    fofyijcond=zeros(N,mquad);
    mu1cond=zeros(N,mquad);
    mu2cond=zeros(N,mquad);
    u=zeros(N,mquad); 
    u1=zeros(N,1);
    for l=1:mquad 
      mu1cond(:,l)=getmu1(alp+(identity(:,l))'*quadvals1,BINN1);
      mu2cond(:,l)=getmu2(blp+(identity(:,l))'*quadvals2,BINN2);
      temp = getf1cond(Y,mu1cond(:,l),BINN1).*p;
      fofyijcond(:,l)= temp+(1-p).*getf2cond(Y,mu2cond(:,l),BINN2);
      cc=(temp+(1-p).*getf2cond(Y,mu2cond(:,l),BINN2));
      u1(cc==0) = zeros(size(cc(cc==0),1),1);
      u1(cc~=0) = temp(cc~=0)./cc(cc~=0);
      u(:,l)=u1;
    end;

    YVAR=[stackrows(u),ones(N*mquad,1)];
    LINKFUNCTION=PLINK;
    XVARS=kron(WMAT,ones(mquad,1));
    ERRDISFUNCTION='binoml';
    OFFSET=zeros(size(YVAR,1),1);
    WEIGHTS=zeros(size(YVAR,1),1);
    count1=0;
    count2=0;
    prodi=zeros(N2,mquad);
    for i=1:N2
      prodi(i,:)=prod(fofyijcond(ID==i,:)); %1 x mquad row vector
      ti=size(ID(ID==i),1);
      for j=1:ti;
        count2=count2+1;
        for l=1:mquad;
          count1=count1+1;
          aa = (prodi(i,:)*quadwts);
          if (aa==0)
             WEIGHTS(count1)=0;
          else
             WEIGHTS(count1)=prodi(i,l)*quadwts(l)/aa;
          end;
        end; % l
      end; % j
    end; % i
    
    [gamma, fits, resids, glmcovgamma, covd, devlist]=glmfit;
    gamma=gamma(:,1);


    % M step for alpha
    warning off
    newmodel; 
    clear global YVAR XVARS;
    global YVAR LINKFUNCTION XVARS ERRDISFUNCTION SCALEPAR INC_CONST;
    global OFFSET WEIGHTS DETAILSFILE NAMEXV NAMELIST NAMEYV;
    warning on
    NAMEXV='[';
    for i=1:(size(XMAT,2)+mquad)
      NAMEXV=strcat(NAMEXV,'A'); 
      NAMEXV=strcat(NAMEXV,int2str(i));
      if (i~=size(XMAT,2))
        NAMEXV=strcat(NAMEXV,',');
      end;
    end;
    NAMEXV=strcat(NAMEXV,']');
    NAMEYV='[y]';
    NAMELIST=['Alpha'];
    for i=1:mquad
      NAMELIST=str2mat(NAMELIST,['Avar ',num2str(i)]);
    end;

    if strcmp(ERRDIST1,'binoml')
      YVAR=kron([Y,BINN1],ones(mquad,1));
    else
      YVAR=kron([Y],ones(mquad,1));
    end;
    LINKFUNCTION=MU1LINK;
    XVARS=[ kron(XMAT,ones(mquad,1)), kron(ones(N,1),identity)];

    ERRDISFUNCTION=ERRDIST1;
    SCALEPAR=1;
    INC_CONST=0;
    OFFSET=kron(OFFSET1,ones(mquad,1));
    WEIGHTS=zeros(size(YVAR,1),1);
    count1=0;
    count2=0;
    for i=1:N2
      ti=size(ID(ID==i),1);
      for j=1:ti;
        count2=count2+1;
        for l=1:mquad;
          count1=count1+1;
          aa = (prodi(i,:)*quadwts);
          if (aa==0)
             WEIGHTS(count1)=0;
          else
             WEIGHTS(count1)=u(count2,l)*prodi(i,l)*quadwts(l)/aa;
          end;         
        end; % l
      end; % j
    end; % i
    
    [alphatmp fits resids glmcovalpha covd devlist]=glmfit;
    alpha=alphatmp(1:size(XMAT,2),1);
    quadvals1=alphatmp(size(XMAT,2)+1:end,1);


    % M step for beta
    warning off
    newmodel;
    clear global YVAR XVARS;
    global YVAR LINKFUNCTION XVARS ERRDISFUNCTION SCALEPAR INC_CONST;
    global OFFSET WEIGHTS DETAILSFILE NAMEXV NAMELIST NAMEYV;
    warning on
    NAMEXV='[';
    for i=1:size(ZMAT,2)+1
      NAMEXV=strcat(NAMEXV,'B');
      NAMEXV=strcat(NAMEXV,int2str(i));
      if (i~=size(ZMAT,2))
        NAMEXV=strcat(NAMEXV,',');
      end;
    end;
    NAMEXV=strcat(NAMEXV,']');
    NAMEYV='[y]';
    NAMELIST=['Beta'];
    for i=1:mquad
      NAMELIST=str2mat(NAMELIST,['Bvar ',num2str(i)]);
    end;

    if strcmp(ERRDIST2,'binoml')
      YVAR=kron([Y,BINN2],ones(mquad,1));
    else
      YVAR=kron([Y],ones(mquad,1));
    end;
    LINKFUNCTION=MU2LINK;
    XVARS=[ kron(ZMAT,ones(mquad,1)), kron(ones(N,1),identity)];
    ERRDISFUNCTION=ERRDIST2;
    SCALEPAR=1;
    INC_CONST=0;
    OFFSET=kron(OFFSET2,ones(mquad,1));
    WEIGHTS=zeros(size(YVAR,1),1);
    count1=0;
    count2=0;
    for i=1:N2
      ti=size(ID(ID==i),1);
      for j=1:ti;
        count2=count2+1;
        for l=1:mquad;
          count1=count1+1;
          aa=(prodi(i,:)*quadwts);
          if (aa==0)
             WEIGHTS(count1)=0;
          else
             WEIGHTS(count1)=(1-u(count2,l))*prodi(i,l)*quadwts(l)/aa;
          end;         
        end; % l
      end; % j
    end; % i
 
    [betatmp fits resids glmcovbeta covd devlist]=glmfit;
    beta=betatmp(1:size(ZMAT,2),1);
    quadvals2=betatmp((size(ZMAT,2)+1:end),1);


    if (its > 5)
       quadwts=quadwtsnew';
    end;

    maxchange=max(abs([alpha-alphaold;beta-betaold;gamma-gammaold]))
    loglik = getloglik(alpha,beta,gamma,mquad,quadwts,quadvals1, quadvals2)
  
    if (maxchange<toler)
      converge=1
      emitcount=5000;
      its=maxits+1;
    end;

    end; % while statement 
end;

%calculate variance_covariance matrix;
% calculate empirical version of Fk;

FEM = fridel(alpha, beta, gamma, Y, mquad, quadvals1,quadvals2, quadwts,dimparm1, dimparm2)

%calculate MC estimate of Fk;    

B=100;
FMC_ft=zeros(dimparm,dimparm);
for t = 1:mquad
    Bt=floor(1+B*quadwts(t));
    lamda1=exp(alp+identity(t,:)*quadvals1);
    lamda2=exp(blp+identity(t,:)*quadvals2);
    sum=zeros(dimparm,dimparm);
    for b = 1:Bt
         z = rand(N,1);
         poi1 = poissrnd(lamda1);
         poi2 = poissrnd( lamda2);
         yijstar = (z <= p).*poi1 + (z > p).*poi2;
         Ft_b=fridel(alpha, beta, gamma, yijstar, mquad, quadvals1, quadvals2, quadwts,dimparm1, dimparm2);
         sum=sum+Ft_b; % sum of all b(Bt);
    end;
    F_t = quadwts(t)*sum / Bt;
    FMC_ft = FMC_ft + F_t;
 end;

Cov_fridel = inv(FMC_ft)*FEM*inv(FMC_ft) 
