function [ft]=fridel(alpha, beta, gamma, y, mquad, quadvals1, quadvals2, quadwts,dimparm1, dimparm2)
% calculate Ft in Fridel formular;

global ID LINKFUNCTION XMAT OFFSET1 OFFSET2 identity;
global N N2 ZMAT WMAT BINN1 BINN2 PLINK MU1LINK MU2LINK ERRDIST1 ERRDIST2;


alp=OFFSET1+XMAT*alpha;
blp=OFFSET2+ZMAT*beta;
glp=WMAT*gamma;
p=getmixp(glp);

mu1cond=zeros(N,mquad);
mu2cond=zeros(N,mquad);
f1ijcond=zeros(N,mquad);
f2ijcond=zeros(N,mquad);
fijcond=zeros(N,mquad);

for k=1:mquad
    mu1cond(:,k)=getmu1(alp + identity(k,:)*quadvals1,BINN1);
    mu2cond(:,k)=getmu2(blp + identity(k,:)*quadvals2,BINN2);
    f1ijcond(:,k) = getf1cond(y,mu1cond(:,k),BINN1);
    f2ijcond(:,k) = getf2cond(y,mu2cond(:,k),BINN1);
    fijcond(:,k)= p.*f1ijcond(:,k)+(1-p).*f2ijcond(:,k);
end;

wik=zeros(1,mquad);
f_dd=zeros(dimparm1,dimparm1); %Fdd is empirical estimate;
f_dv=zeros(dimparm1,dimparm2);
f_vv=zeros(dimparm2,dimparm2);
for i=1:N2
    pi=p(ID==i);
    Wi=WMAT(ID==i,:);
    Xi=XMAT(ID==i,:); 
    Zi=ZMAT(ID==i,:);
    yi=y(ID==i);
    prodi=prod(fijcond(ID==i,:)); %1 x mquad row vector
    wik=prodi.*(quadwts)'/(prodi*quadwts);

    mu1condi = mu1cond(ID==i,:);
    mu2condi = mu2cond(ID==i,:);
    f1ijcondi = f1ijcond(ID==i,:);
    f2ijcondi = f2ijcond(ID==i,:);
    fijcondi = fijcond(ID==i,:);

    ti = size(ID(ID==i),1);
    ws = zeros(dimparm1,1);
    we = zeros(1, dimparm2);
    ee = zeros(dimparm1,1);
    dlogfcondi=zeros(dimparm1,mquad);

    for k = 1:mquad
        for j = 1:ti
            if (fijcondi(j,k)==0)
               dlogfcondi(:,k) = dlogfcondi(:,k) + 0;
               else
               aa1 = 1/fijcondi(j,k);
               if (aa1==Inf)
                   dlogfcondi(:,k) = dlogfcondi(:,k) + 0;
                   else
                   dlogfcondi(:,k) = dlogfcondi(:,k) + (1/fijcondi(j,k))*[(f1ijcondi(j,k)-f2ijcondi(j,k))*pi(j)*(1-pi(j))*Wi(j,:)'; pi(j)*f1ijcondi(j,k)*(yi(j)-mu1condi(j,k))*[Xi(j,:),identity(k,:)]';(1-pi(j))*f2ijcondi(j,k)*(yi(j)-mu2condi(j,k))*[Zi(j,:),identity(k,:)]'];
               end;
             end;       
        end; %end of j;
        ws = ws + wik(k)*dlogfcondi(:,k);
    end;
    we = wik*identity;       
    ee = (wik-quadwts')*identity;
    f_dd = f_dd + ws*ws';      
    f_dv = f_dv + ws*we(1:end-1);
    f_vv = f_vv + ti*(ee(1:end-1))'*ee(1:end-1);         
end;

ft=[f_dd f_dv ; f_dv' f_vv];
 